#!/usr/bin/env python

"""
The class which defines the manipulation of firewall rules
for IPTables
"""

__cvsversion__ = '$Id: IPTablesRules.py,v 1.9 2007/12/06 23:32:20 jay Exp $'
__author__ = 'Jay Deiman; jay@splitstreams.com'
__copyright__ = 'Copyright 2007 Jason Deiman'
__license__ = 'GPLv3'
__all__ = ['IPTablesRules']

import os , re , FWRules , utils

class IPTablesRules(FWRules.FWRules):
    """
    Wrapper for the system calls to 'iptables' for the blocking and
    unblocking of IPs
    """
    def __init__(self , config , fwQ , logger):
        self._banTemplate = '%s -I INPUT 1 -s %s -j DROP'
        self._searchTemplate = '%s --line-numbers -L -v -n'
        self._unbanTemplate = '%s -D INPUT %s'
        FWRules.FWRules.__init__(self , config , fwQ , logger)
        
    def ban(self , ip):
        """
        Adds an iptables DROP rule for the ip on the external 
        interface
        """
        cmd = self._banTemplate % (self._config.RuleCmdPath , ip)
        if utils.DEBUG:
            print cmd
        stdin , stdoStde = os.popen4(cmd)
        stdin.close()
        output = stdoStde.read()
        stdoStde.close()
        if output:
            self._logger.log(output , self._logger.LOG_ERR)
        else:
            self._logger.log('Added firewall rule for %s' % ip)
    
    def unban(self , ip):
        """
        Finds a ban rule for the given IP and removes it
        """
        reg = '^(\d+)\s.*?DROP.*all  -- .*?%s\s+0\.0\.0\.0/0' % ip
        ipregex = re.compile(reg)
        # Get the list of entries from iptables
        cmd = self._searchTemplate % (self._config.RuleCmdPath)
        fh = os.popen(cmd)
        while True:
            line = fh.readline().strip()
            if not line: break
            m = ipregex.match(line)
            if m:
                # We have found the line we want to remove, remove it
                cmd = self._unbanTemplate % (self._config.RuleCmdPath ,
                                             m.group(1))
                stdin , stdoStde = os.popen4(cmd)
                stdin.close()
                output = stdoStde.read()
                stdoStde.close()
                if output:
                    self._logger.log(output , self._logger.LOG_ERR)
                else:
                    self._logger.log('Removed firewall rule for %s' % ip)
                break
        # Clear buffer and return
        fh.read()
        fh.close()