#!/usr/bin/env python

"""
Inherits from Reader and defines the reading methods specifically
for reading directly from a specified logfile.
"""

__cvsversion__ = '$Id: LogReader.py,v 1.10 2007/12/06 23:32:20 jay Exp $'
__author__ = 'Jay Deiman; jay@splitstreams.com'
__copyright__ = 'Copyright 2007 Jason Deiman'
__license__ = 'GPLv3'
__all__ = ['LogReader']

import re , DNS , threading , utils , time , os , stat

class LogReader(threading.Thread):
    def __init__(self , filename , logger , actionQ , reBls , whiteList):
        self._filename = filename
        self._logger = logger
        self._actionQ = actionQ
        fst = os.stat(filename)
        self.isFifo = stat.S_ISFIFO(fst.st_mode)
        self._ipRe = re.compile(r'(?:\d{1,3}\.){3}\d{1,3}')
        self._handle = open(filename)
        if not self.isFifo:
            # Seek to the end of the file
            self._handle.seek(os.path.getsize(filename))
        self._bls = reBls
        self._wls = whiteList
        DNS.ParseResolvConf()
        threading.Thread.__init__(self)
        
    def run(self):
        while True:
            line = ''
            if self.isFifo:
                line = self._handle.readline()
            else:
                line = self._tailReadline()
            for reg in self._bls.getRegexes():
                m = reg.search(line)
                if m:
                    # The required matching hostname can be either
                    # an IP address or a DNS name
                    host = m.group(1)
                    ip = ''
                    if self._ipRe.match(host):
                        ip = host
                    else:
                        ip = self._resolvHostname(host)
                        if not ip:
                            self._logger.log('Invalid hostname: %s' % host , 
                                 self._logger.LOG_ERR)
                            continue
                    if not self._wls.isWhitelisted(ip):
                        self._actionQ.put((ip , line.strip()))
                        if utils.DEBUG:
                            print 'Found: %s -- %s' % (ip , line.strip())
                    else:
                        self._logger.log('Whitelisted IP failed access: %s' %
                                         ip)
                    
    def _resolvHostname(self , host):
        """
        Utility function to resolve a hostname to an IP
        """
        ans = DNS.DnsRequest(name=host , qtype='A').req()
        if len(ans.answers) > 0:
            return ans.answers[0]['data']
        return None
    
    def _tailReadline(self):
        """
        A blocking file tailer.  This will blocker the caller until a line
        has been added, unlike a readline on a normal file which does
        not block
        """
        def reopen(filename):
            while True:
                try:
                    newfh = open(filename)
                except IOError:
                    time.sleep(0.01)
                else:
                    return newfh
        while True:
            line = self._handle.readline()
            if not line:
                if not os.path.samestat(os.stat(self._filename) ,
                                        os.fstat(self._handle.fileno())):
                    self._handle.close()
                    self._handle = reopen(self._filename)
                time.sleep(0.01)
            else:
                return line
    
    def __del__(self):
        if not self._handle.closed:
            self._handle.close()