#!/usr/bin/env python

"""
This class is a wrapper for the 'syslog' facilities to make it easy
to write log messages to the syslog.
"""

__cvsversion__ = '$Id: Logger.py,v 1.4 2007/12/06 23:32:20 jay Exp $'
__author__ = 'Jay Deiman; jay@splitstreams.com'
__copyright__ = 'Copyright 2007 Jason Deiman'
__license__ = 'GPLv3'
__all__ = ['Logger']

import os , sys , syslog

class Logger(object):
    """
    Handle the system logging of messages
    """
    def __init__(self , config):
        self._config = config
        self.logname = os.path.basename(sys.argv[0])
        syslog.openlog(self.logname , self._config.LogOptions ,
                       self._config.LogFacility)
        # Set up a constants mirror of the syslog lib for all the
        # LOG_* constants so that if another class wishes to change
        # priority, they don't have to import syslog for the
        # constants, they can just use instance.LOG_*
        for k in syslog.__dict__:
            if k[:3] == 'LOG':
                exec('self.%s = syslog.%s' % (k , k))
    
    def log(self , mess , priority=None):
        if not priority: priority = self._config.LogPriority
        syslog.syslog(priority , mess)
        
    def __del__(self):
        syslog.closelog()