"""
The class which defines the manipulation of firewall rules
for PF
"""

__cvsversion__ = '$Id: PfRules.py,v 1.11 2007/12/06 23:32:20 jay Exp $'
__author__ = 'Jay Deiman; jay@splitstreams.com'
__copyright__ = 'Copyright 2007 Jason Deiman'
__license__ = 'GPLv3'
__all__ = ['PfRules']

import os , FWRules , re , utils

class PfRules(FWRules.FWRules):
    """
    Wrapper for the system calls to 'pfctl' for the blocking and
    unblocking of IPs
    """
    def __init__(self , config , fwQ , logger):
        """
        Initialize the rule templates and the thread object
        """
        self._banTemplate = '%s -a %s -f -'
        self._flushTemplate = '%s -a %s -F rules'
        self._curRlsTemplate = '%s -a %s -s rules'
        self._killStateTemplate = '%s -k %s'
        FWRules.FWRules.__init__(self , config , fwQ , logger)
        
    def ban(self , ip):
        """
        This is a simple operation.  We just create the ban rule
        and send it to pfctl on stdin
        """
        curRules = self._getCurRules(ip)
        curRules.append('block in from %s to any\n' % ip)
        bCmd = self._banTemplate % (self._config.RuleCmdPath , 
                                    self._config.PfAnchorName)
        if utils.DEBUG:
            print bCmd
        si , sose = os.popen4(bCmd)
        for rule in curRules:
            si.write(rule)
        si.close()
        output = sose.read()
        sose.close()
        if output:
            self._logger.log(output)
        else:
            self._logger.log('Added firewall rule for %s' % ip)
        # Kill any existing state entries
        killState = self._killStateTemplate % (self._config.RuleCmdPath , ip)
        if utils.DEBUG:
            print killState
        si , sose = os.popen4(killState)
        si.close()
        self._logger.log(sose.read())
        sose.close()
    
    def unban(self , ip):
        """
        First we have to get the list of current rules in the
        anchor, removing the one containing the ip we are going
        to remove and then reload them all
        """
        curRules = self._getCurRules(ip)
        flush = self._flushTemplate % (self._config.RuleCmdPath , 
                                       self._config.PfAnchorName)
        if utils.DEBUG:
            print flush
        si , sose = os.popen4(flush)
        si.close()
        output = sose.read()
        sose.close()
        if output.strip() != 'rules cleared':
            self._logger.log(output)
            return
        bCmd = self._banTemplate % (self._config.RuleCmdPath ,
                                    self._config.PfAnchorName)
        if utils.DEBUG:
            print bCmd
        si , sose = os.popen4(bCmd)
        for rule in curRules:
            si.write(rule)
        si.close()
        output = sose.read()
        if output:
            self._logger.log(output)
        else:
            self._logger.log('Removed firewall rule for %s' % ip)
        
    def _getCurRules(self , ip=None):
        """
        Gets the current list of rules.  If 'ip' is given, it returns all
        rules except that IP.
        """
        crCmd = self._curRlsTemplate % (self._config.RuleCmdPath ,
                                        self._config.PfAnchorName)
        if ip != None:
            regex = re.compile('^block.*?from %s' % ip)
        if utils.DEBUG:
            print crCmd
        rh = os.popen(crCmd)
        curRules = []
        while True:
            line = rh.readline()
            if not line: break
            if regex and not regex.match(line): curRules.append(line)
            elif ip == None: curRules.append(line)
        return curRules
