#!/usr/bin/env python

"""
This is a class to handle the whitelisting of IPs
"""

__cvsversion__ = '$Id: Whitelist.py,v 1.5 2007/12/06 23:32:20 jay Exp $'
__author__ = 'Jay Deiman; jay@splitstreams.com'
__copyright__ = 'Copyright 2007 Jason Deiman'
__license__ = 'GPLv3'
__all__ = ['Whitelist']

import re , utils

class Whitelist(object):
    def __init__(self , config , logger):
        self._config = config
        self._logger = logger
        self._ips = {}
        self._networks = []
        self._readWLFile()
        
    def _readWLFile(self):
        fh = open(self._config.WhiteFile)
        comment = re.compile(r'^\s*#.*')
        whitespace = re.compile(r'^\s*$')
        data = re.compile(r'^(\S{7,})')
        lineCount = 0
        while True:
            line = fh.readline()
            lineCount += 1
            if not line: break
            if whitespace.match(line) or comment.match(line): continue
            m = data.match(line)
            if m:
                self._processListing(m.group(1))
            else:
                self._logger.log('Invalid whitelist entry at line %d' % 
                                 lineCount , self._logger.LOG_ERR)
    
    def _processListing(self , rawData):
        if not '/' in rawData:
            self._ips[str(utils.ip2int(rawData))] = True
            return
        ip , mask = rawData.split('/')
        intIp = utils.ip2int(ip)
        intMask = 0
        if '.' in mask:
            intMask = utils.ip2int(mask)
        else:
            intMask = utils.slash2intip(mask)
        self._networks.append({'ip': intIp , 'mask': intMask})
        
    def addWhitelist(self , rawData):
        self._processListing(rawData)
    
    def isWhitelisted(self , ip):
        intIp = utils.ip2int(ip)
        if self._ips.has_key(str(intIp)): return True
        for net in self._networks:
            bcInt = utils.getBCAddrInt(net['ip'] , net['mask'])
            netInt = utils.getNetworkInt(net['ip'] , net['mask'])
            if intIp >= netInt and intIp <= bcInt:
                return True
        return False
            